"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inject = exports.flush = exports.wait = exports.fireEvent = exports.cleanup = exports.render = void 0;
const angular = require("angular");
require("angular-mocks");
const dom_1 = require("@testing-library/dom");
const mountedContainers = new Set();
const mountedScopes = new Set();
function render(ui, { container, baseElement = container, queries, scope, ignoreUnknownElements = false, } = {}) {
    if (!baseElement) {
        // default to document.body instead of documentElement to avoid output of potentially-large
        // head elements (such as JSS style blocks) in debug output
        baseElement = document.body;
    }
    if (!container) {
        container = baseElement.appendChild(document.createElement('div'));
    }
    // we'll add it to the mounted containers regardless of whether it's actually
    // added to document.body so the cleanup method works regardless of whether
    // they're passing us a custom container or not.
    mountedContainers.add(container);
    const $rootScope = inject('$rootScope');
    const $compile = inject('$compile');
    const $scope = $rootScope.$new();
    Object.assign($scope, scope);
    mountedScopes.add($scope);
    const element = $compile(ui)($scope)[0];
    container.appendChild(element);
    $scope.$digest();
    if (!ignoreUnknownElements) {
        assertNoUnknownElements(container);
    }
    return {
        container,
        baseElement,
        debug: (el = baseElement) => Array.isArray(el)
            ? // eslint-disable-next-line no-console
                el.forEach(e => console.log((0, dom_1.prettyDOM)(e)))
            : // eslint-disable-next-line no-console,
                console.log((0, dom_1.prettyDOM)(el)),
        asFragment: () => {
            const source = container;
            /* istanbul ignore if (jsdom limitation) */
            if (typeof document.createRange === 'function') {
                return document.createRange().createContextualFragment(source.innerHTML);
            }
            const template = document.createElement('template');
            template.innerHTML = source.innerHTML;
            return template.content;
        },
        $scope,
        unmount: () => {
            $scope.$destroy();
        },
        ...(0, dom_1.getQueriesForElement)(baseElement, queries),
    };
}
exports.render = render;
function cleanup() {
    mountedScopes.forEach(cleanupScope);
    mountedContainers.forEach(cleanupAtContainer);
}
exports.cleanup = cleanup;
// maybe one day we'll expose this (perhaps even as a utility returned by render).
// but let's wait until someone asks for it.
function cleanupAtContainer(container) {
    if (container.parentNode === document.body) {
        document.body.removeChild(container);
    }
    mountedContainers.delete(container);
}
function cleanupScope(scope) {
    scope.$destroy();
    mountedScopes.delete(scope);
}
function toCamel(s) {
    return s
        .toLowerCase()
        .replace(/-([a-z])/g, (_, letter) => letter.toUpperCase());
}
function assertNoUnknownElements(element) {
    const { tagName } = element;
    if (tagName.includes('-')) {
        const $injector = inject('$injector');
        const directiveName = `${toCamel(tagName)}Directive`;
        if (!$injector.has(directiveName)) {
            throw Error(`Unknown component/directive "${tagName}". Are you missing an import?`);
        }
    }
    Array.from(element.children).forEach(assertNoUnknownElements);
}
function inject(name) {
    let service;
    angular.mock.inject([
        name,
        injected => {
            service = injected;
        },
    ]);
    return service;
}
exports.inject = inject;
// AngularJS maps `mouseEnter` to `mouseOver` and `mouseLeave` to `mouseOut`
// Create a copy so we can alter it
const fireEvent = dom_1.fireEvent.bind(null);
exports.fireEvent = fireEvent;
Object.assign(fireEvent, dom_1.fireEvent);
fireEvent.mouseEnter = dom_1.fireEvent.mouseOver;
fireEvent.mouseLeave = dom_1.fireEvent.mouseOut;
function flush(millis = 50) {
    const $flushPendingTasks = inject('$flushPendingTasks');
    const $rootScope = inject('$rootScope');
    const $interval = inject('$interval');
    $interval.flush(millis);
    $flushPendingTasks(millis);
    $rootScope.$digest();
}
exports.flush = flush;
function wait(callback, options = {}) {
    return (0, dom_1.waitFor)(() => {
        flush(options.interval);
        if (callback) {
            callback();
        }
    }, options);
}
exports.wait = wait;
__exportStar(require("@testing-library/dom"), exports);
